/**
 * @fileOverview The "removeEmptyParagraphs" plugin.
 *
 */
(function(){

    var removeEmptyPFilter = {
        elements : {
            p : function(element){
                var child = element.children[0],
                    pattern = /\bep-gadget\b/;          // regular expression to indicate gadget DIVs by class name

                // is p tag empty?
                if (child && !child.children && (!child.value || CKEDITOR.tools.trim( child.value ).match( /^(?:&nbsp;|\xa0|<br \/>)$/ ))){
                    // is previous or next element a gadget div -> remove empty p tag
                    if((element.previous && element.previous.attributes && element.next && pattern.test(element.previous.attributes['class'])) || (element.next && element.next.attributes && pattern.test(element.next.attributes['class']))){
                        return false;
                    }

                    // is previous or next element a table element -> remove empty p tag
                    if((element.previous && element.previous.name && element.previous.name==="table" && element.next) || (element.next && element.next.name && element.next.name==="table")){
                        return false;
                    }

                    // please don't remove this comment (because there is a logic depending on gadgets in table cells)
                    // if(element.parent && element.parent.name && element.parent.name==="td" && (element.previous && element.previous.attributes && pattern.test(element.previous.attributes['class']) || !element.next)){
                    //     return false;
                    // }

                    // remove all empty p-tags in table cells
                    if(element.parent && element.parent.name && element.parent.name==="td"){
                        return false;
                    }
                }

                return element;
            }
        }
    };

    CKEDITOR.plugins.add('epremoveemptyparagraph', {
        init: function(editor){
            var dataProcessor = editor.dataProcessor,
                dataFilter = dataProcessor && dataProcessor.dataFilter,
                htmlFilter = dataProcessor && dataProcessor.htmlFilter;

            if(dataFilter){
                // give the filters lower priority makes it get applied after the default ones
                dataFilter.addRules(removeEmptyPFilter, 100);
            }

            if(htmlFilter){
                // give the filters lower priority makes it get applied after the default ones
                htmlFilter.addRules(removeEmptyPFilter, 100);
            }
        },
        afterInit: function(editor){
            var dataProcessor = editor.dataProcessor,
                htmlFilter = dataProcessor && dataProcessor.htmlFilter;

            if(htmlFilter){
                // give the filters lower priority makes it get applied after the default ones
                htmlFilter.addRules(removeEmptyPFilter, 100);
            }
        }
    });

})();